/*
The software in this package is distributed under the GNU General
Public License version 2 (with a special exception described below).

A copy of GNU General Public License (GPL) is included in this distribution,
in the file COPYING.GPL.

As a special exception, if other files instantiate templates or use macros
or inline functions from this file, or you compile this file and link it
with other works to produce a work based on this file, this file
does not by itself cause the resulting work to be covered
by the GNU General Public License.

However the source code for this file must still be made available
in accordance with section (3) of the GNU General Public License.

This exception does not invalidate any other reasons why a work based
on this file might be covered by the GNU General Public License.
*/
/***************************************************************************
 *   Copyright (C) 2004 by Intra2net AG                                    *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <string>
#include <sstream>
#include <stdexcept>

#define BOOST_TEST_MAIN
#define BOOST_TEST_DYN_LINK
#include <boost/test/unit_test.hpp>

#include <ipfunc.hxx>

namespace std
{

template <class charT, class traits>
inline std::basic_ostream<charT, traits>& operator<<(std::basic_ostream<charT, traits>& os,
                                                     const std::vector<IP_RANGE>& x)
{
    bool is_first = true;
    for (std::vector<IP_RANGE>::const_iterator i=x.begin(); i!=x.end(); i++)
    {
        if (!is_first)
            os << ", ";
        os << i->to_string();
        is_first = false;
    }

    return os;
}

template <class charT, class traits>
inline std::basic_ostream<charT, traits>& operator<<(std::basic_ostream<charT, traits>& os,
                                                     const IP_RANGE& x)
{
    os << x.to_string();
    return os;
}

} // eo namespace std


using namespace std;

BOOST_AUTO_TEST_SUITE(ip_range)

BOOST_AUTO_TEST_CASE(ConstructIP1)
{
    IP_RANGE i("192.168.1.1");
    BOOST_CHECK_EQUAL(string("192.168.1.1"),i.to_string());
}

BOOST_AUTO_TEST_CASE(ConstructIP2)
{
    IP_RANGE i(ip_type::IP,"192.168.1.1");
    BOOST_CHECK_EQUAL(string("192.168.1.1"),i.to_string());
}

BOOST_AUTO_TEST_CASE(ConstructNet1)
{
    IP_RANGE i("192.168.1.1/255.255.255.0");
    BOOST_CHECK_EQUAL(string("192.168.1.0/255.255.255.0"),i.to_string());
}

BOOST_AUTO_TEST_CASE(ConstructNet2)
{
    IP_RANGE i(ip_type::NETWORK,"192.168.1.1","255.255.255.0");
    BOOST_CHECK_EQUAL(string("192.168.1.0/255.255.255.0"),i.to_string());
}

BOOST_AUTO_TEST_CASE(ConstructNet3)
{
    IP_RANGE i("172.16.0.0/16");
    BOOST_CHECK_EQUAL(string("172.16.0.0/255.255.0.0"),i.to_string());
}

BOOST_AUTO_TEST_CASE(ConstructRange1)
{
    IP_RANGE i("192.168.1.5-192.168.3.1");
    BOOST_CHECK_EQUAL(string("192.168.1.5-192.168.3.1"),i.to_string());
}

BOOST_AUTO_TEST_CASE(ConstructRange2)
{
    IP_RANGE i(ip_type::RANGE,"192.168.1.5","192.168.3.1");
    BOOST_CHECK_EQUAL(string("192.168.1.5-192.168.3.1"),i.to_string());
}

BOOST_AUTO_TEST_CASE(ConstructRangeSwap)
{
    IP_RANGE i(ip_type::RANGE,"192.168.3.1","192.168.1.5");
    BOOST_CHECK_EQUAL(string("192.168.1.5-192.168.3.1"),i.to_string());
}

BOOST_AUTO_TEST_CASE(OverlapIP)
{
    IP_RANGE a("192.168.1.5");
    IP_RANGE b("192.168.1.5");

    BOOST_CHECK_EQUAL(true,a.overlapping(b));
}

BOOST_AUTO_TEST_CASE(OverlapNet1)
{
    IP_RANGE a("192.168.1.5/255.255.255.0");
    IP_RANGE b("192.168.1.6");

    BOOST_CHECK_EQUAL(true,a.overlapping(b));
}

BOOST_AUTO_TEST_CASE(OverlapNet2)
{
    IP_RANGE a("192.168.1.5/255.255.255.0");
    IP_RANGE b("192.168.1.44/255.255.255.248");

    BOOST_CHECK_EQUAL(true,a.overlapping(b));
}

BOOST_AUTO_TEST_CASE(OverlapNet3)
{
    IP_RANGE a("192.168.1.5/255.255.255.0");
    IP_RANGE b("192.168.0.0/255.255.0.0");

    BOOST_CHECK_EQUAL(true,a.overlapping(b));
}

BOOST_AUTO_TEST_CASE(OverlapNet4)
{
    IP_RANGE a("192.169.1.5/255.255.255.0");
    IP_RANGE b("192.168.0.0/255.255.0.0");

    BOOST_CHECK_EQUAL(false,a.overlapping(b));
}

BOOST_AUTO_TEST_CASE(OverlapRange1)
{
    IP_RANGE a("192.168.1.5-192.168.1.50");
    IP_RANGE b("192.168.1.6");

    BOOST_CHECK_EQUAL(true,a.overlapping(b));
}

BOOST_AUTO_TEST_CASE(OverlapRange2)
{
    IP_RANGE a("192.168.1.5-192.168.1.50");
    IP_RANGE b("192.168.1.0/255.255.255.128");

    BOOST_CHECK_EQUAL(true,a.overlapping(b));
}

BOOST_AUTO_TEST_CASE(OverlapRange3)
{
    IP_RANGE a("192.168.1.5-192.168.1.50");
    IP_RANGE b("192.168.0.0/255.255.0.0");

    BOOST_CHECK_EQUAL(true,a.overlapping(b));
}

BOOST_AUTO_TEST_CASE(OverlapRange4)
{
    IP_RANGE a("192.168.1.5-192.168.1.50");
    IP_RANGE b("192.168.1.1-192.168.1.5");

    BOOST_CHECK_EQUAL(true,a.overlapping(b));
}

BOOST_AUTO_TEST_CASE(OverlapRange5)
{
    IP_RANGE a("192.168.1.5-192.168.1.50");
    IP_RANGE b("192.168.1.1-192.168.1.4");

    BOOST_CHECK_EQUAL(false,a.overlapping(b));
}

BOOST_AUTO_TEST_CASE(OverlapRange6)
{
    IP_RANGE a("192.168.1.5-192.168.1.50");
    IP_RANGE b("192.168.1.1-192.168.2.4");

    BOOST_CHECK_EQUAL(true,a.overlapping(b));
}

BOOST_AUTO_TEST_CASE(OverlapRange7)
{
    IP_RANGE a("192.168.1.5-192.168.1.50");
    IP_RANGE b("192.168.1.6-192.168.1.7");

    BOOST_CHECK_EQUAL(true,a.overlapping(b));
}

BOOST_AUTO_TEST_CASE(WithinNet1)
{
    IP_RANGE a("192.168.1.0/255.255.255.0");
    IP_RANGE b("192.168.1.6");

    BOOST_CHECK_EQUAL(false,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinNet2)
{
    IP_RANGE a("192.168.1.6");
    IP_RANGE b("192.168.1.0/255.255.255.0");

    BOOST_CHECK_EQUAL(true,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinNet3)
{
    IP_RANGE a("192.168.1.6/255.255.255.248");
    IP_RANGE b("192.168.1.0/255.255.255.0");

    BOOST_CHECK_EQUAL(true,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinNet4)
{
    IP_RANGE a("192.168.0.0/255.255.0.0");
    IP_RANGE b("192.168.1.0/255.255.255.0");

    BOOST_CHECK_EQUAL(false,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinNet5)
{
    IP_RANGE a("192.168.1.0/255.255.255.0");
    IP_RANGE b("192.168.1.0/255.255.255.0");

    BOOST_CHECK_EQUAL(true,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinNet6)
{
    IP_RANGE a("192.168.1.0/255.255.255.0");
    IP_RANGE b("192.168.1.0/255.255.255.128");

    BOOST_CHECK_EQUAL(false,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinNet7)
{
    IP_RANGE a("192.168.1.0/255.255.255.128");
    IP_RANGE b("192.168.1.0/255.255.255.0");

    BOOST_CHECK_EQUAL(true,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinRange1)
{
    IP_RANGE a("192.168.1.0/255.255.255.0");
    IP_RANGE b("192.168.1.0-192.168.1.255");

    BOOST_CHECK_EQUAL(true,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinRange2)
{
    IP_RANGE a("192.168.1.0");
    IP_RANGE b("192.168.1.0-192.168.1.255");

    BOOST_CHECK_EQUAL(true,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinRange3)
{
    IP_RANGE a("192.168.1.1-192.168.1.1");
    IP_RANGE b("192.168.1.0-192.168.1.255");

    BOOST_CHECK_EQUAL(true,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinRange4)
{
    IP_RANGE a("192.168.1.0-192.168.1.2");
    IP_RANGE b("192.168.1.1-192.168.1.255");

    BOOST_CHECK_EQUAL(false,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinRange5)
{
    IP_RANGE a("192.168.1.199-192.168.1.202");
    IP_RANGE b("192.168.1.1-192.168.1.200");

    BOOST_CHECK_EQUAL(false,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinRange6)
{
    IP_RANGE a("192.168.1.0-192.168.1.202");
    IP_RANGE b("192.168.1.1-192.168.1.200");

    BOOST_CHECK_EQUAL(false,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinRange7)
{
    IP_RANGE a("192.168.1.0-192.168.1.202");
    IP_RANGE b("192.168.1.0/255.255.255.0");

    BOOST_CHECK_EQUAL(true,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinRange8)
{
    IP_RANGE a("192.168.0.0-192.168.1.202");
    IP_RANGE b("192.168.1.0/255.255.255.0");

    BOOST_CHECK_EQUAL(false,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinRange9)
{
    IP_RANGE a("192.168.0.0-192.168.2.202");
    IP_RANGE b("192.168.1.0/255.255.255.0");

    BOOST_CHECK_EQUAL(false,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(WithinRange10)
{
    IP_RANGE a("192.168.1.10-192.168.2.0");
    IP_RANGE b("192.168.1.0/255.255.255.0");

    BOOST_CHECK_EQUAL(false,a.is_within(b));
}

BOOST_AUTO_TEST_CASE(OperatorPlus1)
{
    IP_RANGE a("192.168.2.0/255.255.255.0");

    a=a+256;

    BOOST_CHECK_EQUAL(string("192.168.3.0/255.255.255.0"),a.to_string());
}

BOOST_AUTO_TEST_CASE(OperatorPlus2)
{
    IP_RANGE a("192.168.2.0/255.255.255.0");

    a=a+13;

    BOOST_CHECK_EQUAL(string("192.168.2.13-192.168.3.12"),a.to_string());
}

BOOST_AUTO_TEST_CASE(OperatorPlusOverflow)
{
    IP_RANGE a("255.255.255.0/255.255.255.0");
    BOOST_REQUIRE_THROW(a=a+1, std::range_error);
}

BOOST_AUTO_TEST_CASE(OperatorMinus)
{
    IP_RANGE a("192.168.2.0/255.255.255.0");
    IP_RANGE b("192.168.1.0/255.255.255.0");

    BOOST_CHECK_EQUAL(256,a-b);
}

BOOST_AUTO_TEST_CASE(NetmaskToBits1)
{
    IP_RANGE a("255.255.255.255");
    int bits = a.calc_netmask_bits(a.get_base());
    BOOST_CHECK_EQUAL(32,bits);
}

BOOST_AUTO_TEST_CASE(NetmaskToBits2)
{
    IP_RANGE a("255.255.255.0");
    int bits = a.calc_netmask_bits(a.get_base());
    BOOST_CHECK_EQUAL(24,bits);
}

BOOST_AUTO_TEST_CASE(NetmaskToBits3)
{
    IP_RANGE a("255.255.0.0");
    int bits = a.calc_netmask_bits(a.get_base());
    BOOST_CHECK_EQUAL(16,bits);
}

BOOST_AUTO_TEST_CASE(NetmaskToBits4)
{
    IP_RANGE a("255.0.0.0");
    int bits = a.calc_netmask_bits(a.get_base());
    BOOST_CHECK_EQUAL(8,bits);
}
BOOST_AUTO_TEST_CASE(Range2Cidr1)
{
    IP_RANGE range("192.168.1.100-192.168.1.227");
    vector<IP_RANGE> cidrs = range.to_cidr();

    vector<IP_RANGE> check_cidrs;
    check_cidrs.push_back("192.168.1.100/30");
    check_cidrs.push_back("192.168.1.104/29");
    check_cidrs.push_back("192.168.1.112/28");
    check_cidrs.push_back("192.168.1.128/26");
    check_cidrs.push_back("192.168.1.192/27");
    check_cidrs.push_back("192.168.1.224/30");

    BOOST_CHECK_EQUAL(check_cidrs,cidrs);
}

BOOST_AUTO_TEST_CASE(Range2Cidr2)
{
    IP_RANGE range("0.0.0.0-255.255.255.255");
    vector<IP_RANGE> cidrs = range.to_cidr();

    vector<IP_RANGE> check_cidrs;
    check_cidrs.push_back("0.0.0.0/0");

    BOOST_CHECK_EQUAL(check_cidrs,cidrs);
}

BOOST_AUTO_TEST_CASE(Range2Cidr3)
{
    IP_RANGE range("192.168.1.3");
    vector<IP_RANGE> cidrs = range.to_cidr();

    vector<IP_RANGE> check_cidrs;
    check_cidrs.push_back("192.168.1.3/32");

    BOOST_CHECK_EQUAL(check_cidrs,cidrs);
}

BOOST_AUTO_TEST_CASE(Range2Cidr4)
{
    IP_RANGE range("172.16.0.0/16");
    vector<IP_RANGE> cidrs = range.to_cidr();

    vector<IP_RANGE> check_cidrs;
    check_cidrs.push_back("172.16.0.0/16");

    BOOST_CHECK_EQUAL(check_cidrs,cidrs);
}

BOOST_AUTO_TEST_CASE(Range2Cidr5)
{
    IP_RANGE range(ip_type::NETWORK, "172.16.0.0", "255.255.0.0");
    vector<IP_RANGE> cidrs = range.to_cidr();

    vector<IP_RANGE> check_cidrs;
    check_cidrs.push_back("172.16.0.0/16");

    BOOST_CHECK_EQUAL(check_cidrs,cidrs);
}
BOOST_AUTO_TEST_CASE(Range2Cidr6)
{
    IP_RANGE range("192.168.1.0-192.168.2.0");
    vector<IP_RANGE> cidrs = range.to_cidr();

    vector<IP_RANGE> check_cidrs;
    check_cidrs.push_back("192.168.1.0/24");
    check_cidrs.push_back("192.168.2.0/32");

    BOOST_CHECK_EQUAL(check_cidrs,cidrs);
}

BOOST_AUTO_TEST_CASE(Range2Cidr7)
{
    IP_RANGE range("0.0.0.1-255.255.255.254");
    vector<IP_RANGE> cidrs = range.to_cidr();

    vector<IP_RANGE> check_cidrs;
    check_cidrs.push_back("0.0.0.1/32");
    check_cidrs.push_back("0.0.0.2/31");
    check_cidrs.push_back("0.0.0.4/30");
    check_cidrs.push_back("0.0.0.8/29");
    check_cidrs.push_back("0.0.0.16/28");
    check_cidrs.push_back("0.0.0.32/27");
    check_cidrs.push_back("0.0.0.64/26");
    check_cidrs.push_back("0.0.0.128/25");
    check_cidrs.push_back("0.0.1.0/24");
    check_cidrs.push_back("0.0.2.0/23");
    check_cidrs.push_back("0.0.4.0/22");
    check_cidrs.push_back("0.0.8.0/21");
    check_cidrs.push_back("0.0.16.0/20");
    check_cidrs.push_back("0.0.32.0/19");
    check_cidrs.push_back("0.0.64.0/18");
    check_cidrs.push_back("0.0.128.0/17");
    check_cidrs.push_back("0.1.0.0/16");
    check_cidrs.push_back("0.2.0.0/15");
    check_cidrs.push_back("0.4.0.0/14");
    check_cidrs.push_back("0.8.0.0/13");
    check_cidrs.push_back("0.16.0.0/12");
    check_cidrs.push_back("0.32.0.0/11");
    check_cidrs.push_back("0.64.0.0/10");
    check_cidrs.push_back("0.128.0.0/9");
    check_cidrs.push_back("1.0.0.0/8");
    check_cidrs.push_back("2.0.0.0/7");
    check_cidrs.push_back("4.0.0.0/6");
    check_cidrs.push_back("8.0.0.0/5");
    check_cidrs.push_back("16.0.0.0/4");
    check_cidrs.push_back("32.0.0.0/3");
    check_cidrs.push_back("64.0.0.0/2");
    check_cidrs.push_back("128.0.0.0/2");
    check_cidrs.push_back("192.0.0.0/3");
    check_cidrs.push_back("224.0.0.0/4");
    check_cidrs.push_back("240.0.0.0/5");
    check_cidrs.push_back("248.0.0.0/6");
    check_cidrs.push_back("252.0.0.0/7");
    check_cidrs.push_back("254.0.0.0/8");
    check_cidrs.push_back("255.0.0.0/9");
    check_cidrs.push_back("255.128.0.0/10");
    check_cidrs.push_back("255.192.0.0/11");
    check_cidrs.push_back("255.224.0.0/12");
    check_cidrs.push_back("255.240.0.0/13");
    check_cidrs.push_back("255.248.0.0/14");
    check_cidrs.push_back("255.252.0.0/15");
    check_cidrs.push_back("255.254.0.0/16");
    check_cidrs.push_back("255.255.0.0/17");
    check_cidrs.push_back("255.255.128.0/18");
    check_cidrs.push_back("255.255.192.0/19");
    check_cidrs.push_back("255.255.224.0/20");
    check_cidrs.push_back("255.255.240.0/21");
    check_cidrs.push_back("255.255.248.0/22");
    check_cidrs.push_back("255.255.252.0/23");
    check_cidrs.push_back("255.255.254.0/24");
    check_cidrs.push_back("255.255.255.0/25");
    check_cidrs.push_back("255.255.255.128/26");
    check_cidrs.push_back("255.255.255.192/27");
    check_cidrs.push_back("255.255.255.224/28");
    check_cidrs.push_back("255.255.255.240/29");
    check_cidrs.push_back("255.255.255.248/30");
    check_cidrs.push_back("255.255.255.252/31");
    check_cidrs.push_back("255.255.255.254/32");

    BOOST_CHECK_EQUAL(check_cidrs,cidrs);
}

BOOST_AUTO_TEST_CASE(Substract1)
{
    IP_RANGE range("192.168.1.1-192.168.1.100");
    set<IP_RANGE> to_sub;
    to_sub.insert(IP_RANGE("192.168.1.3"));

    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;
    check_range.push_back("192.168.1.1-192.168.1.2");
    check_range.push_back("192.168.1.4-192.168.1.100");

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(Substract2)
{
    IP_RANGE range("192.168.1.1-192.168.1.100");
    set<IP_RANGE> to_sub;
    to_sub.insert(IP_RANGE("192.168.1.1"));

    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;
    check_range.push_back("192.168.1.2-192.168.1.100");

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(Substract3)
{
    IP_RANGE range("192.168.1.1-192.168.1.100");
    set<IP_RANGE> to_sub;
    to_sub.insert(IP_RANGE("192.168.1.1"));
    to_sub.insert(IP_RANGE("192.168.1.61"));
    to_sub.insert(IP_RANGE("192.168.1.50-192.168.1.60"));

    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;
    check_range.push_back("192.168.1.2-192.168.1.49");
    check_range.push_back("192.168.1.62-192.168.1.100");

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(Substract4)
{
    IP_RANGE range("192.168.1.1-192.168.1.100");
    set<IP_RANGE> to_sub;
    to_sub.insert(IP_RANGE("192.168.1.50-192.168.1.100"));

    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;
    check_range.push_back("192.168.1.1-192.168.1.49");

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(Substract5)
{
    IP_RANGE range("192.168.1.1-192.168.1.100");
    set<IP_RANGE> to_sub;
    to_sub.insert(IP_RANGE("192.168.1.50-192.168.1.99"));

    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;
    check_range.push_back("192.168.1.1-192.168.1.49");
    check_range.push_back("192.168.1.100-192.168.1.100");

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(Substract6)
{
    IP_RANGE range("192.168.1.1-192.168.1.100");
    set<IP_RANGE> to_sub;
    to_sub.insert(IP_RANGE("192.168.1.2-192.168.1.100"));

    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;
    check_range.push_back("192.168.1.1");

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(Substract7)
{
    IP_RANGE range("192.168.1.1-192.168.1.100");
    set<IP_RANGE> to_sub;
    to_sub.insert(IP_RANGE("192.168.1.1-192.168.1.100"));
    to_sub.insert(IP_RANGE("192.168.1.50-192.168.1.61"));
    to_sub.insert(IP_RANGE("192.168.1.61"));
    to_sub.insert(IP_RANGE("192.168.1.62"));
    to_sub.insert(IP_RANGE("192.168.1.63"));

    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(Substract8)
{
    IP_RANGE range("192.168.1.1-192.168.1.100");
    set<IP_RANGE> to_sub;
    to_sub.insert(IP_RANGE("192.168.1.10-192.168.1.20"));
    to_sub.insert(IP_RANGE("192.168.1.11-192.168.1.30"));
    to_sub.insert(IP_RANGE("192.168.1.1-192.168.1.3"));
    to_sub.insert(IP_RANGE("192.168.1.8"));
    to_sub.insert(IP_RANGE("192.168.1.9"));

    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;
    check_range.push_back("192.168.1.4-192.168.1.7");
    check_range.push_back("192.168.1.31-192.168.1.100");

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(Substract9)
{
    IP_RANGE range("192.168.1.1-192.168.1.100");
    set<IP_RANGE> to_sub;
    to_sub.insert(IP_RANGE("192.168.1.9-192.168.1.15"));
    to_sub.insert(IP_RANGE("192.168.1.10-192.168.1.20"));

    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;
    check_range.push_back("192.168.1.1-192.168.1.8");
    check_range.push_back("192.168.1.21-192.168.1.100");

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(Substract10)
{
    IP_RANGE range("192.168.1.1-192.168.1.2");
    set<IP_RANGE> to_sub;
    to_sub.insert(IP_RANGE("192.168.1.1"));

    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;
    check_range.push_back("192.168.1.2");

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(Substract11)
{
    IP_RANGE range("192.168.1.1-192.168.1.2");
    set<IP_RANGE> to_sub;
    to_sub.insert(IP_RANGE("192.168.1.2"));

    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;
    check_range.push_back("192.168.1.1");

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(Substract12)
{
    IP_RANGE range("192.168.1.1");
    set<IP_RANGE> to_sub;
    to_sub.insert(IP_RANGE("192.168.1.1"));

    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(Substract13)
{
    IP_RANGE range("192.168.1.1");

    set<IP_RANGE> to_sub;
    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;
    check_range.push_back("192.168.1.1");

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(Substract14)
{
    IP_RANGE range("192.168.1.1-192.168.1.100");
    set<IP_RANGE> to_sub;

    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;
    check_range.push_back("192.168.1.1-192.168.1.100");

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(Substract15)
{
    IP_RANGE range("192.168.1.1-192.168.1.100");
    set<IP_RANGE> to_sub;
    to_sub.insert(IP_RANGE("192.168.1.1-192.168.1.60"));
    to_sub.insert(IP_RANGE("192.168.1.62-192.168.1.100"));

    vector<IP_RANGE> new_range = range.substract(to_sub);

    vector<IP_RANGE> check_range;
    check_range.push_back("192.168.1.61");

    BOOST_CHECK_EQUAL(check_range,new_range);
}

BOOST_AUTO_TEST_CASE(IpNumString1)
{
    IP_RANGE myip("192.168.1.50");
    string expected("192168001050");

    BOOST_CHECK_EQUAL(expected,IP_RANGE::ip_num_string(myip.get_base()));
}

BOOST_AUTO_TEST_CASE(IpNumString2)
{
    IP_RANGE myip("0.0.0.0");
    string expected("000000000000");

    BOOST_CHECK_EQUAL(expected,IP_RANGE::ip_num_string(myip.get_base()));
}

BOOST_AUTO_TEST_CASE(IpNumString3)
{
    IP_RANGE myip("255.255.255.255");
    string expected("255255255255");

    BOOST_CHECK_EQUAL(expected,IP_RANGE::ip_num_string(myip.get_base()));
}

BOOST_AUTO_TEST_CASE(NetmapValidation1)
{
    IP_RANGE myip("192.168.1.11");
    IP_RANGE original("192.168.1.0/255.255.255.0");
    IP_RANGE target("192.168.2.0/255.255.0.0");

    BOOST_CHECK_EQUAL(false,myip.netmap(original,target));
}

BOOST_AUTO_TEST_CASE(NetmapValidation2)
{
    IP_RANGE myip("192.168.3.11");
    IP_RANGE original("192.168.1.0/255.255.255.0");
    IP_RANGE target("192.168.2.0/255.255.255.0");

    BOOST_CHECK_EQUAL(false,myip.netmap(original,target));
}

BOOST_AUTO_TEST_CASE(NetmapValidation3)
{
    IP_RANGE myip("192.168.1.70-192.168.2.20");
    IP_RANGE original("192.168.1.0/255.255.255.0");
    IP_RANGE target("192.168.2.0/255.255.255.0");

    BOOST_CHECK_EQUAL(false,myip.netmap(original,target));
}

BOOST_AUTO_TEST_CASE(Netmap1)
{
    IP_RANGE myip("192.168.1.11");
    IP_RANGE original("192.168.1.0/255.255.255.0");
    IP_RANGE target("192.168.2.0/255.255.255.0");

    BOOST_CHECK_EQUAL(true,myip.netmap(original,target));
    BOOST_CHECK_EQUAL(IP_RANGE("192.168.2.11"),myip);
}

BOOST_AUTO_TEST_CASE(Netmap2)
{
    IP_RANGE myip("192.168.1.11-192.168.1.22");
    IP_RANGE original("192.168.1.0/255.255.255.0");
    IP_RANGE target("192.168.2.0/255.255.255.0");

    BOOST_CHECK_EQUAL(true,myip.netmap(original,target));
    BOOST_CHECK_EQUAL(IP_RANGE("192.168.2.11-192.168.2.22"),myip);
}

BOOST_AUTO_TEST_CASE(Netmap3)
{
    IP_RANGE myip("192.168.1.0/255.255.255.0");
    IP_RANGE original("192.168.1.0/255.255.255.0");
    IP_RANGE target("192.168.2.0/255.255.255.0");

    BOOST_CHECK_EQUAL(true,myip.netmap(original,target));
    BOOST_CHECK_EQUAL(IP_RANGE("192.168.2.0/255.255.255.0"),myip);
}

BOOST_AUTO_TEST_CASE(ResolveDNS)
{
    IP_RANGE test;
    BOOST_CHECK_EQUAL("127.0.0.1", test.resolve_ip("localhost"));
}

/*
// Note: Disabled on the Intranator
BOOST_AUTO_TEST_CASE(ResolveDNSIPv6)
{
    IP_RANGE test;
    BOOST_CHECK_EQUAL("::1", test.resolve_ip("localhost", true));
}
*/

BOOST_AUTO_TEST_CASE(ResolveExistingIP)
{
    IP_RANGE test;
    BOOST_CHECK_EQUAL("192.168.1.254", test.resolve_ip("192.168.1.254"));
}


BOOST_AUTO_TEST_SUITE_END()
